==> hexdebug-common-0.2.2%2B1.20.1-20250123.173024-62-sources.jar: gay/object/hexdebug/mixin/MixinStaffCastEnv.java <==
package gay.object.hexdebug.mixin;

import at.petrak.hexcasting.api.casting.eval.ResolvedPattern;
import at.petrak.hexcasting.api.casting.eval.env.StaffCastEnv;
import at.petrak.hexcasting.api.casting.math.HexCoord;
import at.petrak.hexcasting.api.mod.HexStatistics;
import at.petrak.hexcasting.common.msgs.MsgNewSpellPatternC2S;
import gay.object.hexdebug.items.EvaluatorItem;
import gay.object.hexdebug.registry.HexDebugItems;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

import java.util.HashSet;
import java.util.List;
import net.minecraft.class_1792;
import net.minecraft.class_3222;

@Mixin(StaffCastEnv.class)
public abstract class MixinStaffCastEnv {
    @Inject(method = "handleNewPatternOnServer", at = @At("HEAD"), cancellable = true)
    private static void handleNewEvaluatorPatternOnServer$hexdebug(
        class_3222 sender,
        MsgNewSpellPatternC2S msg,
        CallbackInfo ci
    ) {
        var item = sender.method_5998(msg.handUsed()).method_7909();
        if (item != HexDebugItems.EVALUATOR.getValue()) return;

        // FIXME: copied from handleNewPatternOnServer because I can't figure out how to inject before getStaffcastVM

        boolean cheatedPatternOverlap = false;

        List<ResolvedPattern> resolvedPatterns = msg.resolvedPatterns();
        if (!resolvedPatterns.isEmpty()) {
            var allPoints = new HashSet<HexCoord>();
            for (int i = 0; i < resolvedPatterns.size() - 1; i++) {
                ResolvedPattern pat = resolvedPatterns.get(i);
                allPoints.addAll(pat.getPattern().positions(pat.getOrigin()));
            }
            var currentResolvedPattern = resolvedPatterns.get(resolvedPatterns.size() - 1);
            var currentSpellPoints = currentResolvedPattern.getPattern()
                .positions(currentResolvedPattern.getOrigin());
            if (currentSpellPoints.stream().anyMatch(allPoints::contains)) {
                cheatedPatternOverlap = true;
            }
        }

        if (cheatedPatternOverlap) {
            ci.cancel();
            return;
        }

        sender.method_7281(HexStatistics.PATTERNS_DRAWN);

        // actual new logic

        EvaluatorItem.handleNewPatternOnServer(sender, msg);
        ci.cancel();
    }
}

==> hexdebug-common-0.7.0%2B1.20.1-20251022.035706-16-sources.jar: gay/object/hexdebug/mixin/MixinStaffCastEnv.java <==
package gay.object.hexdebug.mixin;

import at.petrak.hexcasting.api.casting.eval.ResolvedPattern;
import at.petrak.hexcasting.api.casting.eval.env.StaffCastEnv;
import at.petrak.hexcasting.api.casting.math.HexCoord;
import at.petrak.hexcasting.api.mod.HexStatistics;
import at.petrak.hexcasting.common.msgs.MsgNewSpellPatternC2S;
import gay.object.hexdebug.items.EvaluatorItem;
import gay.object.hexdebug.registry.HexDebugItems;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

import java.util.HashSet;
import java.util.List;
import net.minecraft.class_1792;
import net.minecraft.class_3222;

@Mixin(StaffCastEnv.class)
public abstract class MixinStaffCastEnv {
    @Inject(method = "handleNewPatternOnServer", at = @At("HEAD"), cancellable = true)
    private static void handleNewEvaluatorPatternOnServer$hexdebug(
        class_3222 sender,
        MsgNewSpellPatternC2S msg,
        CallbackInfo ci
    ) {
        var item = sender.method_5998(msg.handUsed()).method_7909();
        if (!(item instanceof EvaluatorItem)) return;

        // FIXME: copied from handleNewPatternOnServer because I can't figure out how to inject before getStaffcastVM

        boolean cheatedPatternOverlap = false;

        List<ResolvedPattern> resolvedPatterns = msg.resolvedPatterns();
        if (!resolvedPatterns.isEmpty()) {
            var allPoints = new HashSet<HexCoord>();
            for (int i = 0; i < resolvedPatterns.size() - 1; i++) {
                ResolvedPattern pat = resolvedPatterns.get(i);
                allPoints.addAll(pat.getPattern().positions(pat.getOrigin()));
            }
            var currentResolvedPattern = resolvedPatterns.get(resolvedPatterns.size() - 1);
            var currentSpellPoints = currentResolvedPattern.getPattern()
                .positions(currentResolvedPattern.getOrigin());
            if (currentSpellPoints.stream().anyMatch(allPoints::contains)) {
                cheatedPatternOverlap = true;
            }
        }

        if (cheatedPatternOverlap) {
            ci.cancel();
            return;
        }

        sender.method_7281(HexStatistics.PATTERNS_DRAWN);

        // actual new logic

        EvaluatorItem.handleNewPatternOnServer(sender, msg);
        ci.cancel();
    }
}
